import requests, json, logging

class Configuration:
    def metadata(self):
        try:
            r = requests.get('http://169.254.169.254/latest/dynamic/instance-identity/document', timeout=1)
            return r.json()
        except requests.exceptions.ConnectionError:
            logging.error("Simulation mode enabled.")
            return {
                  "privateIp" : "10.123.26.236",
                  "devpayProductCodes" : None,
                  "availabilityZone" : "ap-southeast-2a",
                  "version" : "2010-08-31",
                  "region" : "ap-southeast-2",
                  "instanceId" : "i-04ec586a0e91b640c",
                  "instanceType" : "t2.medium",
                  "pendingTime" : "2017-02-13T05:57:33Z",
                  "imageId" : "ami-1aefee79",
                  "architecture" : "x86_64",
                  "kernelId" : None,
                  "ramdiskId" : None
                }



import boto3, logging, sys
import ec2

class Base:
    options = None
    metadata = None
    session = None
    ec2 = None

    def __init__(self, options, metadata):
        self.options = options
        self.metadata = metadata
        self.session = boto3.Session(region_name=metadata['region'])
        self.ec2 = ec2.Ec2(self.session)

    def attach(self):
        name = self.ec2.get_instance_name(self.metadata['instanceId']) or self.metadata['instanceId']
        volume_name = "%s-%s" % (name, self.options.device)
        logging.info("Volume name: %s" % volume_name)

        logging.info("Finding volume...")

        volume_id = self.ec2.get_latest_volume_id_available(self.options.uuid)
        if volume_id:
            logging.info("Volume found: %s" % volume_id)
            logging.info("Checking volume is in same availability zone as instance...")
            if self.ec2.get_volume_region(volume_id) != self.metadata['availabilityZone']:
                logging.info("Volume in another availability zone, snapshot required.")
                snapshot_id = ec2.create_snapshot(volume_id)
                if snapshot_id:
                    logging.info("Snapshot created: %s" % snapshot_id)
                    volume_id = None
        else:
            snapshot_id = self.ec2.get_latest_snapshot_id(self.options.uuid)
            if snapshot_id:
                logging.info("Snapshot found: %s" % snapshot_id)
            else:
                logging.info("No snapshot found. An empty #%s volume will be created of #%s GB." % (self.options.type, self.options.size))

        if not volume_id:
            logging.info("Creating volume...")
            volume_id = self.ec2.create_volume(self.options.size, self.options.type, self.metadata['availabilityZone'], snapshot_id)
            if volume_id:
                logging.info("Created volume: %s" % volume_id)

                logging.info("Tagging volume...")
                if self.ec2.tag_volume(volume_id, volume_name, self.options):
                    logging.info("Volume tagged.")
                else:
                    logging.warning("Volume failed tagging.")
            else:
                logging.error("Volume failed creation.")
                sys.exit(1)

        if volume_id:
            logging.info("Attaching volume...")
            if self.ec2.attach_volume(volume_id, self.metadata['instanceId'], self.options.device):
                logging.info('Volume attached to instance.')
            else:
                logging.info('Volume attachment failed.')
                sys.exit(1)

    def snapshot(self):
        logging.info("Finding volumes...")
        volumes = self.ec2.get_volume_id(self.metadata['instanceId'], self.options.uuid)

        if len(volumes) > 0:
            for volume_id in volumes:
                logging.info("Creating snapshot for volume %s" % volume_id)
                if self.ec2.create_snapshot(volume_id, self.options.tags):
                    logging.info("Volume %s snapshot created." % volume_id)
                else:
                    logging.error("Volume %s snapshot failed." % volume_id)
        else:
            logging.info("No volumes found")

    # TODO test this method - should work?
    def tag(self):
        logging.info("Finding volumes...")
        volumes = self.ec2.get_volume_id(self.metadata['instanceId'], self.options.uuid)

        if len(volumes) > 0:
            for volume_id in volumes:
                if self.ec2.tag_volume(volume_id, self.ec2.get_volume_name(volume_id), self.options):
                    logging.info("Volume %s tagged." % volume_id)
                else:
                    logging.error("Volume %s failed tagging." % volume_id)
        else:
            logging.info("No volumes found")





















            #



from pip.req import parse_requirements
from setuptools import setup

setup(
    name='ebs-pin',
    version='0.3.2',
    scripts=['ebs-pin'],
    packages=['ebspin'],
    url='https://github.com/discobean/ebs-pin',
    description='Pin EBS volumes in a multi-az EC2 instance',
    install_requires=[
        'appdirs>=1.4.0',
        'boto3>=1.4.4',
        'botocore>=1.5.10',
        'docutils>=0.13.1',
        'futures>=3.0.5',
        'jmespath>=0.9.1',
        'packaging>=16.8',
        'pyparsing>=2.1.10',
        'python-dateutil>=2.6.0',
        'requests>=2.13.0',
        's3transfer>=0.1.10',
        'six>=1.10.0'],
    keywords='ebs ebspin ebs-pin'
)




import time, logging, sys

class Ec2:
    session = None
    client = None

    def __init__(self, session):
        self.session = session
        self.client = self.session.client('ec2')

    def get_latest_volume_id_available(self, uuid):
        filters = [
                { "Name": 'tag-key',   "Values": [ 'UUID' ] },
                { "Name": 'tag-value', "Values": [ uuid ] }
            ]
        try:
            volumes = self.client.describe_volumes(Filters=filters)['Volumes']
            volumes = sorted(volumes, key=lambda ss:ss['CreateTime'])
            volume = [x for x in volumes if x['State'] == 'available'].pop()
            return volume['VolumeId']
        except:
            return None

    def get_latest_snapshot_id(self, uuid):
        filters = [
                { 'Name': 'tag-key',   'Values': ['UUID'] },
                { 'Name': 'tag-value', 'Values': [ uuid ] },
                { 'Name': 'status',    'Values': ['completed'] }
            ]

        try:
            snapshots = self.client.describe_snapshots(Filters=filters)['Snapshots']
            snapshot = sorted(snapshots, key=lambda ss:ss['StartTime']).pop()
            return snapshot['SnapshotId']
        except:
            return None

    def get_instance_name(self, instance_id):
        filters = [
                { "Name": 'resource-id', "Values": [ instance_id ] },
                { "Name": 'key',         "Values": [ 'Name' ] }
            ]

        try:
            result = self.client.describe_tags(Filters=filters)['Tags'][0]['Value']
            return result
        except:
            return None

    def get_volume_id(self, instance_id, uuid):
        filters = [
                { 'Name': 'attachment.instance-id', 'Values': [instance_id] },
                { 'Name': 'tag:UUID', 'Values': [uuid] },
            ]

        try:
            result = self.client.describe_volumes(Filters=filters)
            # return a list of volume_ids
            volumes = [v['VolumeId'] for v in result['Volumes']]
            return volumes
        except:
            return None

    def get_volume_name(self, volume_id):
        attachment = self.client.describe_volumes(VolumeIds=[volume_id])['Volumes'][0]['Attachments'][0]
        instance_name = self.get_instance_name(attachment['InstanceId'])

        return "%s-%s" % (instance_name, attachment['Device'])

    def get_volume_region(self, volume_id):
        try:
            return self.client.describe_volumes(VolumeIds=[volume_id])['Volumes'][0]['AvailabilityZone']
        except:
            return None

    def create_volume(self, size, volume_type, availability_zone, snapshot_id=None):
        try:
            if snapshot_id:
                response = self.client.create_volume(
                    Size=size,
                    SnapshotId=snapshot_id,
                    AvailabilityZone=availability_zone,
                    VolumeType=volume_type
                )
            else:
                response = self.client.create_volume(
                    Size=size,
                    AvailabilityZone=availability_zone,
                    VolumeType=volume_type
                )

            num = 0
            while True:
                if num >= 12:
                    break

                state = self.client.describe_volumes(VolumeIds=[response['VolumeId']])['Volumes'][0]['State']

                if state != 'available':
                    time.sleep(5)
                    num += 1
                    continue

                return response['VolumeId']
        except:
            logging.exception("Failed to create volume")
            return None


    def create_snapshot(self, volume_id, extra_tags=None):
        try:
            snapshot_id = self.client.create_snapshot(VolumeId=volume_id)['SnapshotId']
            tags = self.client.describe_volumes(VolumeIds=[volume_id])['Volumes'][0]['Tags']

            if extra_tags:
                for key, value in extra_tags.iteritems():
                    tags.append({'Key':key, 'Value':value})

            self.tag_snapshot(snapshot_id, tags)

            num = 0
            while True:
                if num >= 12:
                    break

                state = self.client.describe_snapshots(SnapshotIds=[snapshot_id])['Snapshots'][0]['State']

                if state != 'completed':
                    time.sleep(5)
                    num += 1
                    continue

                return snapshot_id

        except:
            logging.exception("Failed to create snapshot")
            return None


    def tag_volume(self, volume_id, volume_name, options):
        try:
            tags = [
                    { 'Key': 'Name',         'Value': volume_name },
                    { 'Key': 'UUID',         'Value': options.uuid }
                ]

            tags = [x for x in tags if x['Value'] is not None]

            # Add the tags provided from the command line
            for key, value in options.tags.iteritems():
                tags.append({'Key':key, 'Value':value})

            return self.client.create_tags(
                    Resources=[ volume_id ],
                    Tags=tags
                )
        except:
            logging.exception("Failed to create tags")
            return None


    def tag_snapshot(self, snapshot_id, tags):
        try:
            return self.client.create_tags(
                Resources=[snapshot_id],
                Tags=tags
            )
        except:
            logging.exception("Failed to create_tags %s" % tags)
            return None

    def attach_volume(self, volume_id, instance_id, device):
        try:
            response = self.client.attach_volume(
                VolumeId=volume_id,
                InstanceId=instance_id,
                Device=device
            )

            num = 0
            while True:
                if num >= 12:
                    break

                try:
                    state = self.client.describe_volumes(VolumeIds=[volume_id])['Volumes'][0]['Attachments'][0]['State']
                except:
                    logging.exception("Error getting state")

                if state != 'attached':
                    time.sleep(5)
                    num += 1
                    continue

                return volume_id
        except:
            logging.exception("Failed to mount volume")
            return None
